#include <Arduino.h>
#include <ArduinoLog.h>

#include "config.h"
#include "pins.h"
#include "nfc.h"
#include "otaproxy.h"
#include "helpers.h"
#include "Desfire.h"

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

#include <WiFiClient.h>
#include <PubSubClient.h>
#include <SPI.h>
#include <SSD1306Wire.h>

WiFiClient espClient;
//Config_Data config;

PubSubClient* mqtt;
NFC* nfc;
OTAProxy* ota;

void setup_wifi() 
{
    delay(10);
    Serial.println("Connecting Wifi ...");

    WiFi.mode(WIFI_STA);
    WiFi.begin(WLAN_SSID, WLAN_PASS);

    while (WiFi.status() != WL_CONNECTED)
    {
        delay(500);
    }

    randomSeed(micros());
    Serial.println("WiFi connected");
}

void reconnect() 
{
    while (!mqtt->connected())
    {
        String clientId = "FabReader_";
        clientId += String(FABREADERID);

        Serial.println("Connecting MQTT ...");
        if (mqtt->connect(clientId.c_str()))
        {
            Serial.println("MQTT connected");
            char id[6] = "00000";
            sprintf(id, "%05d", FABREADERID);
            mqtt->publish("fabreader", id);

            char topic_requestOTA[] = "fabreader/00000/requestOTA";
            sprintf(topic_requestOTA, "fabreader/%05d/requestOTA", FABREADERID);
            mqtt->subscribe(topic_requestOTA);

            char topic_stopOTA[] = "fabreader/00000/stopOTA";
            sprintf(topic_stopOTA, "fabreader/%05d/stopOTA", FABREADERID);
            mqtt->subscribe(topic_stopOTA);
        } else {
            Serial.print("failed, rc=");
            Serial.print(mqtt->state());
            Serial.println(" try again in 5 seconds");
            delay(5000);
        }
    }
}

void callback(char* topic, byte* payload, unsigned int length) 
{
    Serial.println("Receive Message");
    Serial.println(topic);
    if(ota->hasActiveOTA())
    {
        ota->continueOTA(topic, payload, length);
    }
}

void setup()
{
    Serial.begin(115200);
    Serial.print("\n\n\n");
    Serial.println("Booting ...");
    
    pinMode(PIN_BUZZER, OUTPUT);
    pinMode(PIN_BUTTON, INPUT);
    pinMode(PIN_ETH_SPI_SS, OUTPUT);
    digitalWrite(PIN_ETH_SPI_SS, HIGH);

    Serial.println("Connecting NFC ...");
    nfc = new NFC(PIN_RFID_SPI_SS, PIN_RFID_RST);
    if(!(nfc->rfid->PCD_PerformSelfTest()))
    {
        Serial.println("NFC Test failed");
    }
    Serial.println("NFC connected");

    // setup_wifi();

    // mqtt = new PubSubClient(espClient);
    // mqtt->setServer(MQTT_BROKER, 1883);
    // mqtt->setCallback(callback);

    // ota = new OTAProxy(mqtt, nfc, FABREADERID);
}

void loop()
{
    // if(!(nfc->hasCardSelected()) && nfc->hasNewCard() && nfc->selectCard())
    // {
    //     // DESFire::mifare_desfire_tag tag;
    //     // tag.pcb = 0x0A;
    //     // tag.cid = 0x00;
    //     // DESFire::mifare_desfire_aid_t aid;

    //     // aid.data[0] = 0x42;
    //     // aid.data[1] = 0x41;
    //     // aid.data[2] = 0x46;

    //     // DESFire::StatusCode status = nfc->rfid->MIFARE_DESFIRE_SelectApplication(&tag, &aid);
    //     // Serial.printf("%02x\n", status.mfrc522);
        
    //     // char command_h[63*2] = "905a00000342414600";
    //     // byte command[63] = { 0 };
    //     // byte command_len = strlen(command_h)/2;
        
        
    //     // byte response[63] = { 0 };
    //     // byte response_len = 0;
    //     // char response_h[63*2] = { 0 };

    //     // Serial.print("Command: ");
    //     // Serial.println(command_h);

    //     // chars2bytes(command_h, command, true);
    //     // MFRC522::StatusCode status = nfc->Transceive(command, command_len, response, &response_len);
    //     // Serial.printf("PICC_Tranceive: 0x%02x\n", status);
    //     // bytes2chars(response, response_len, response_h, true);

    //     // Serial.print("Response: ");
    //     // Serial.println(response_h);
    // }
    // if(nfc->hasCardSelected() && nfc->isCardLost())
    // {
    //     nfc->deselectCard();
    // }
    
    if( nfc->hasCardSelected() ) {
        if( nfc->isCardLost() ) {
            nfc->deselectCard();
            Serial.println("Stop");
        }
    }
    else {
        if( nfc->hasNewCard() ) {
            if( nfc->selectCard() ) {
                Serial.println("Run");
            }
        }
    }

    /*
    if(!(nfc->hasCardSelected()) && nfc->hasNewCard())
    {
        if(nfc->selectCard())
        {
            Serial.println("Run");
        }
    }

    if(nfc->hasCardSelected() && nfc->isCardLost())
    {
        Serial.println("Stop");
    }
    */

    // if (!mqtt->connected())
    // {
    //     reconnect();
    // }
    // mqtt->loop();

    // if(ota->hasActiveOTA() && nfc->isCardLost())
    // {
    //     ota->cancelOTA();
    // }

    // if(!ota->hasActiveOTA() && nfc->hasNewCard())
    // {
    //     ota->startOTA();
    // }
}